@@ -21,9 +21,9 @@ sns.set(style="darkgrid")
 
 __author__ = "Dylan Gatlin"
 
-# The offsets given by the filters + observed errors
-camera_offsets = np.array([0., -85.3, -60.93, 0, 67.99, 11.85])
-
+camera_offsets = np.array([0.0, -50.2985, -41.05837499999998,
+                           11.466562500000009, 52.425531914893625, 0.0])
+camera_shifts = np.array([0., 0., +0.05, 0., -0.02])
 
 def build_filt(obj_arr: np.ndarray):
     ecc = np.sqrt(obj_arr['a']**2 - obj_arr['b']**2) / obj_arr['a']
@@ -36,11 +36,12 @@ def build_filt(obj_arr: np.ndarray):
         & (obj_arr["cpeak"] < 60000)
         # & (obj_arr["npix"] > 5)
         & (ecc < 0.7)
-        )
+    )
     # print(obj_arr["a"] * 0.216, obj_arr["b"] * 0.216,
     # np.mean([obj_arr["a"], obj_arr["b"]]))
     return filt
 
+
 def show_img(obj_arr: np.ndarray, obj_filt, data_sub: np.ndarray, plot_file=""):
     fig, ax = plt.subplots(1, 1, figsize=(8, 8))
     m, s = np.mean(data_sub), np.std(data_sub)
@@ -61,9 +62,32 @@ def show_img(obj_arr: np.ndarray, obj_filt, data_sub: np.ndarray, plot_file=""):
         fig.savefig(plot_file)
 
 
+def get_img_path(mjd: int, cam_num: int, exp_num: int):
+    """Resolves a path of a gfa image, if none can be found, it will return a
+    path object 'Null'
+    """
+    p = (sdss_paths.gcam / f"{mjd}/proc-gimg-gfa{cam_num:.0f}n-"
+         f"{exp_num:0>4.0f}.fits")
+    try:
+        p.exists()
+    except OSError:
+        time.sleep(1)
+    if not p.exists():
+        p = (sdss_paths.gcam / f"{mjd}/proc-gimg-gfa{cam_num:.0f}n-"
+             f"{exp_num:0>4.0f}.fits.gz")
+    if not p.exists():
+        p = (sdss_paths.gcam / f"{mjd}/gimg-gfa{cam_num:.0f}n-"
+             f"{exp_num:0>4.0f}.fits")
+    if not p.exists():
+        p = (sdss_paths.gcam / f"{mjd}/gimg-gfa{cam_num:.0f}n-"
+             f"{exp_num:0>4.0f}.fits.gz")
+    if not p.exists():
+        p = Path("Null")
+    return p
+
 
 class GFASet:
-    def __init__(self, verbose=False):
+    def __init__(self, verbose: bool = False):
         self.paths = []
         self.filter = []
         self.isots = []
@@ -75,7 +99,7 @@ class GFASet:
         self.ani_fig = None
         self.ani_ax = None
 
-    def add_index(self, paths_iter, im_num):
+    def add_index(self, paths_iter, im_num: int):
         paths = []
         filter = []
         focus = []
@@ -105,7 +129,7 @@ class GFASet:
         self.fwhms.append(fwhms)
         self.n_objs.append(n_objs)
         self.isots.append(isots)
-    
+
     def remove_first_index(self):
         self.paths.pop(0)
         self.filter.pop(0)
@@ -116,8 +140,17 @@ class GFASet:
         self.im_nums.pop(0)
 
     @staticmethod
-    def get_fwhm(path: Path, verbose: bool=False):
-        data = fitsio.read(path, 1)
+    def get_fwhm(path: Path, verbose: bool = False):
+        try:
+            path.exists()
+        except OSError:
+            pass
+        try:
+            data = fitsio.read(path, 1)
+        except OSError:
+            # For cases when the file is actively being written
+            time.sleep(1)
+            data = fitsio.read(path, 1)
         bkg = sep.Background(data.astype(float))
         objs = sep.extract(data - bkg, 1.5, bkg.globalrms)
         filt = build_filt(objs)
@@ -127,8 +160,8 @@ class GFASet:
             # 2.0*np.log(2)*(objs['a'][filt]**2 + objs['b'][filt]**2))
         # )
         fwhm = 0.216 * np.mean(
-             2 * np.sqrt(np.log(2) * (objs['a'][filt]**2 + objs['b'][filt]**2))
-            )
+            2 * np.sqrt(np.log(2) * (objs['a'][filt]**2 + objs['b'][filt]**2))
+        )
         if fwhm > 5:
             fwhm = np.nan
         return fwhm, filt.sum(), objs
@@ -155,7 +188,7 @@ class GFASet:
             print("Measurements in arcseconds with a place scale of 0.216")
         print(f"{'Img #':<6} {'Focus':<5} {'GFA1':<6} {'GFA2':<6} {'GFA3':<6}"
               f" {'GFA4':<6}"
-            f" {'GFA5':<6} {'GFA6':<6} {'N Objs':<6}")
+              f" {'GFA5':<6} {'GFA6':<6} {'N Objs':<6}")
         print('=' * 80)
         for i, (im, foc) in enumerate(zip(self.aim_nums, self.afocuses)):
             if np.isnan(foc[1]):
@@ -167,23 +200,27 @@ class GFASet:
             else:
                 n_objs = f"{np.nanmean(self.an_objs[i]):>6.0f}"
             print(f"{im:>6} {foc:>5} "
-                + " ".join([f"{f:>6.2f}"
-                    if not np.isnan(f) else f"{'-':>6}"
-                    for (j, f) in enumerate(self.afwhms[i])]
-                ) + f" {n_objs:6>}"
-            )
-    
+                  + " ".join([f"{f:>6.2f}"
+                              if not np.isnan(f) else f"{'-':>6}"
+                              for (j, f) in enumerate(self.afwhms[i])]
+                             ) + f" {n_objs:6>}"
+                  )
+
     @staticmethod
-    def quadratic(x, a, b, c): 
+    def quadratic(x: float, a: float, b: float, c: float):
         return a * x**2 + x * b + c
 
-    def plot(self, plot_file, fig=None, ax=None):
+    def plot(self, plot_file, fig=None, ax=None, dont_print=False):
         flat_foc = (self.afocuses + camera_offsets).flatten()
         flat_fwhms = self.afwhms.flatten()
         flat_nstars = self.an_objs.flatten()
         nan_filt = ~np.isnan(flat_fwhms)
+        if flat_nstars[nan_filt].size == 0:
+            if not dont_print:
+                print("Nothing to plot")
+            return
         weight_nstars = flat_nstars[nan_filt] / np.nanmax(flat_nstars)
-        weight_times = (10 - (np.nanmax(self.aisots)
+        weight_times = (20 - (np.nanmax(self.aisots)
                               - self.aisots.flatten()[nan_filt]
                               ) * 24 * 60)
         weight_times[weight_times < 0] = 0
@@ -193,9 +230,15 @@ class GFASet:
         a, b, c = np.polyfit(flat_foc[nan_filt],
                              flat_fwhms[nan_filt], deg=2,
                              w=weight)
+        expected = self.quadratic(flat_foc[nan_filt], a, b, c)
+        chi_squared = (expected - flat_fwhms[nan_filt])**2 / expected
+        chi_rejects = chi_squared > np.percentile(chi_squared, 85)
+        a, b, c = np.polyfit(flat_foc[nan_filt][~chi_rejects],
+                             flat_fwhms[nan_filt][~chi_rejects],
+                             deg=2, w=weight[~chi_rejects])
         focs = np.linspace(flat_foc[nan_filt].min(),
                            flat_foc[nan_filt].max(), 100)
-        fit = -b / 2 /a
+        fit = -b / 2 / a
         fwhm = self.quadratic(fit, a, b, c)
         mum = "\u03BCm"
         if a < 0:  # Not a parabola with a minimum
@@ -203,28 +246,34 @@ class GFASet:
             print("Optimal focus not found")
         else:
             fit_found = True
-            print(f'Optimal focus is at {fit:>4.0f}{mum} with FWHM {fwhm:>4.1f}"')
+            if not dont_print:
+                print(f'Optimal focus is at {fit:>4.0f}{mum} with FWHM'
+                      f' {fwhm:>4.1f}"')
         if fig is None:
             for_ani = False
             fig, ax = plt.subplots(1, 1, figsize=(6, 4))
         else:
             for_ani = True
-            
+
         if fit_found:
             ax.plot(focs, self.quadratic(focs, a, b, c),
                     alpha=0.8,
-                    label="Best Fit")
+                    # label="Best Fit"
+                    )
             ax.set_title(f"Best Focus is {-b / 2 / a:.0f}{mum} with FWHM"
                          f' {fwhm:.1f}"')
             ax.axvline(-b / 2 / a, c="r", linestyle="--", alpha=0.6)
-            
+
         not_old = 10 - (np.nanmax(self.aisots) - self.aisots) > 0
         for i in range(6):
             ax.scatter(self.afocuses[not_old[:, i], i] + camera_offsets[i],
                        self.afwhms[not_old[:, i], i], s=6,
-                       alpha=0.8,
-            label=f"GFA {i+1}")
-        ax.legend()
+                       alpha=0.7,
+                       label=f"{i+1}")
+        ax.scatter(flat_foc[nan_filt][chi_rejects],
+                   flat_fwhms[nan_filt][chi_rejects], marker="x", alpha=0.7,
+                   linewidth=0.5, c="k")
+        ax.legend(ncol=6, loc=1)
         ax.set_xlabel("Focus ($\mu m$)")
         ax.set_ylabel("FWHM (arcseconds)")
         if not for_ani:
@@ -232,7 +281,7 @@ class GFASet:
             if plot_file:
                 fig.savefig(plot_file, dpi=100)
         return fig
-            
+
     def separate_plot(self, plot_file=None):
         fig, ax = plt.subplots(1, 1, figsize=(6, 4))
         xs = np.linspace(np.nanmin(self.afocuses), np.nanmax(self.afocuses))
@@ -254,15 +303,15 @@ class GFASet:
         if plot_file:
             fig.savefig(plot_file, dpi=100)
         return fig
-    
+
     def exp_num_plot(self, plot_file=None):
         fig, ax = plt.subplots(1, 1, figsize=(6, 4))
         for i in range(6):
             ax.plot(self.aim_nums, self.afwhms[:, i], linewidth=1,
-                            alpha=0.8,
+                    alpha=0.8,
                     label=f"GFA {i+1}")
             ax.scatter(self.aim_nums, self.afwhms[:, i], s=6,
-                alpha=0.8)
+                       alpha=0.8)
         ax.legend()
         ax.set_xlabel("Exposure Number")
         ax.set_ylabel("FWHM (arcseconds)")
@@ -270,49 +319,60 @@ class GFASet:
         if plot_file:
             fig.savefig(plot_file, dpi=100)
         return fig
-    
+
     def init_continuous_plot(self):
         today = sjd.sjd()
         img_dir = sdss_paths.gcam / f"{today:.0f}/"
         latest = 0
-        for fil in img_dir.glob("proc-gimg-gfa4n-*.fits"):
+        try:
+            img_dir.exists()
+        except OSError:
+            pass
+        for fil in img_dir.glob("gimg-gfa4n-*.fits"):
+            if "snap" in fil.name:
+                continue
             current_num = int(fil.name.split("gfa4n-")[-1].split(".fits")[0])
             latest = max(latest, current_num)
         for im_num in tqdm.tqdm(range(latest - 20, latest + 1)):
             im_ps = []
             for n in range(1, 7):
-                p = img_dir / f"proc-gimg-gfa{n:.0f}n-{im_num:0>4.0f}.fits"
+                p = get_img_path(today, n, im_num)
                 im_ps.append(p)
             self.add_index(im_ps, im_num)
         self.sort()
         self.ani_fig, self.ani_ax = plt.subplots(1, 1, figsize=(6, 4))
-        return self.plot(None, self.ani_fig, self.ani_ax)
-        
+        return self.plot(None, self.ani_fig, self.ani_ax,
+                         dont_print=not self.verbose)
+
     def continuous_plot_update(self, i):
         today = sjd.sjd()
-        img_dir = sdss_paths.gcam / f"{today:.0f}/"
-        im_num = self.aim_nums[-1] + 1
+        im_num = self.im_nums[-1] + 1
         im_num_0 = im_num
-        p = img_dir / f"proc-gimg-gfa4n-{im_num:0>4.0f}.fits"
+        p = get_img_path(today, 4, im_num)
         current_range = self.im_nums[-1] - self.im_nums[0]
+        if self.verbose:
+            print(im_num, p.exists(), im_num_0)
+            print(current_range)
         while p.exists() and (im_num - im_num_0 < current_range):
             im_ps = []
             for n in range(1, 7):
-                p = img_dir / f"proc-gimg-gfa{n:.0f}n-{im_num:0>4.0f}.fits"
+                p = get_img_path(today, n, im_num)
                 im_ps.append(p)
             self.add_index(im_ps, im_num)
             self.remove_first_index()
             im_num += 1
         self.sort()
         self.ani_ax.clear()
-        print(f"Plotting from {self.im_nums[0]:>3.0f}-{self.im_nums[-1]:>3.0f}"
-              ", ", end="")
-        return self.plot(None, fig=self.ani_fig, ax=self.ani_ax)
+        if self.verbose:
+            print(f"Plotting from {self.im_nums[0]:>3.0f}-{self.im_nums[-1]:>3.0f}"
+                  ", ", end="")
+        return self.plot(None, fig=self.ani_fig, ax=self.ani_ax,
+                         dont_print=not self.verbose)
 
 
 def parse_args():
     parser = argparse.ArgumentParser(description="Prints a table of FWHMs from"
-        " GFAs, useful for focus sweeps")
+                                     " GFAs, useful for focus sweeps")
     parser.add_argument("-c", "--continuous", action="store_true",
                         help="Update plot/fit continuously, overrides all other"
                         " plots")
@@ -326,8 +386,12 @@ def parse_args():
                         help="A dash separated beginning and end of the window."
                              " Always include a --master-field with your window."
                              " Ex: -w 15-50 -s 35")
+    parser.add_argument("-i", "--ignore", nargs="+", type=int, default=[],
+                        help="Image numbers to ignore in a window")
     parser.add_argument("-e", "--exp-num", action="store_true",
                         help="Plot fwhm relative to exposure number")
+    parser.add_argument("-t", "--interval", default=15,
+                        help="Gap between reading images when using -c")
     # parser.add_argument("-s", "--master-field", type=int,
     #                     help="The master field with all stars you want. Only"
     #                     " used in conjunction with --window. You need both"
@@ -341,8 +405,8 @@ def parse_args():
     parser.add_argument("-s", "--separate", action="store_true",
                         help="Creates separate fits for each gfa in a plot")
     parser.add_argument("-d", "--plot-image", action="store_true",
-        help="Plot an the sky image with ellipses traced, only works with -f"
-    )
+                        help="Plot an the sky image with ellipses traced, only works with -f"
+                        )
     args = parser.parse_args()
 
     if args.continuous and args.window:
@@ -365,9 +429,9 @@ def main(args=None):
                 fwhm, n_objs, objs = gfas.get_fwhm(file)
                 filt = build_filt(objs)
                 ecc = (np.sqrt(objs['a'][filt]**2 - objs['b'][filt]**2)
-                    / objs['a'][filt])
+                       / objs['a'][filt])
                 print(f"{file.name:<20} {fwhm:>6.2f} {filt.sum():>5.0f}"
-                    f"/{len(objs):<5.0f}  {np.mean(ecc):>6.2f}")
+                      f"/{len(objs):<5.0f}  {np.mean(ecc):>6.2f}")
                 if args.plot_image:
                     data = fitsio.read(file, 1)
                     bkg = sep.Background(data.astype(float))
@@ -376,7 +440,7 @@ def main(args=None):
                 file = Path(file)
                 for p in file.parent.glob(file.name):
                     pass
-                
+
     if args.continuous:
         anis = []
         print("Plotting continuously")
@@ -385,17 +449,17 @@ def main(args=None):
         # animation to run. By rights, it should not be necessary. However,
         # it is apparently essential to the existence of an animation loop
         anis.append(animation.FuncAnimation(gfas.ani_fig,
-            gfas.continuous_plot_update,
-                                interval=1000))
+                                            gfas.continuous_plot_update,
+                                            interval=1000 * args.interval))
         plt.show()
-        
+
     elif not args.window and not args.continuous:
         path_stem = sdss_paths.gcam / f"{args.mjd}"
         low = 1000
         high = 0
         for file in path_stem.glob("gimg-gfa5n-*.fits*"):
             ind = int(file.name.split("-")[-1].rstrip(
-                ".fits").rstrip(".fits.gz"))
+                ".fits.gz"))
             if ind < low:
                 low = ind
             if ind > high:
@@ -403,7 +467,6 @@ def main(args=None):
         if high > low:
             args.window = f"{low:.0f}-{high:.0f}"
 
-    
     if args.window:
         low, high = args.window.split("-")
         try:
@@ -415,23 +478,11 @@ def main(args=None):
         if args.verbose:
             print(f"Reading Images from {low} to {high} in MJD {args.mjd}")
         for im_num in tqdm.tqdm(range(low, high + 1)):
+            if im_num in args.ignore:
+                continue
             im_ps = []
             for n in range(1, 7):
-                p = (sdss_paths.gcam / f"{args.mjd}/proc-gimg-gfa{n:.0f}n-"
-                     f"{im_num:0>4.0f}.fits")
-                try:
-                    p.exists()
-                except OSError:
-                    pass
-                if not p.exists():
-                    p = (sdss_paths.gcam / f"{args.mjd}/proc-gimg-gfa{n:.0f}n-"
-                         f"{im_num:0>4.0f}.fits.gz")
-                if not p.exists():
-                    p = (sdss_paths.gcam / f"{args.mjd}/gimg-gfa{n:.0f}n-"
-                         f"{im_num:0>4.0f}.fits")
-                if not p.exists():
-                    p = (sdss_paths.gcam / f"{args.mjd}/gimg-gfa{n:.0f}n-"
-                         f"{im_num:0>4.0f}.fits.gz")
+                p = get_img_path(args.mjd, n, im_num)
                 im_ps.append(p)
 
             gfas.add_index(im_ps, im_num)
@@ -444,7 +495,7 @@ def main(args=None):
             gfas.exp_num_plot(args.plot_file)
         elif args.plot:
             gfas.plot(args.plot_file)
-            
+
     return 0
 
 
