@@ -16,6 +16,7 @@ In order to run it locally, you will need to either have access to /data, or
  /data/apogee/archive. You'll also need setup local forwarding for InfluxDB
 """
 import argparse
+import multiprocessing
 import sys
 import textwrap
 import warnings
@@ -26,13 +27,13 @@ try:
     from bin import m4l, telescope_status, get_dust
 except ImportError as e:
     raise ImportError('Please add ObserverTools/bin to your PYTHONPATH:'
-        '\n    {}'.format(e))
+                      '\n    {}'.format(e))
 from pathlib import Path
 from tqdm import tqdm
 from astropy.time import Time
 
 from sdssobstools import apogee_data, log_support, boss_data, sdss_paths
-    
+
 from bin import sjd
 
 if sys.version_info.major < 3:
@@ -48,6 +49,7 @@ __version__ = '3.7.5'
 ap_dir = sdss_paths.ap_archive
 b_dir = sdss_paths.boss
 
+
 class Logging:
     """
     A tool to produce a ton of various outputs used for logging. This tool uses
@@ -92,7 +94,7 @@ class Logging:
         self.ap_data = {'dDesign': [], "dConfig": [], 'dTime': [],
                         'iTime': [], 'iID': [],
                         'iSeeing': [], 'iDetector': [], 'iDither': [],
-                        'iNRead': [], 'iEType': [], 'iDesign': [], 
+                        'iNRead': [], 'iEType': [], 'iDesign': [],
                         "iDesign": [], "iConfig": [],
                         'fDesign': [], 'fTime': [], 'fMissing': [], 'fFaint': [],
                         'fNMissing': [], 'fNFaint': [], 'fRatio': [], 'aTime': [],
@@ -108,16 +110,16 @@ class Logging:
         # for BOSS dithers, and AB for APOGEE dithers, dt for boss exposure
         # time, since some boss carts use shorter exposures. All these are
         # combined to fill Summary in self.count_dithers
-        self.design_data = {'cNAPA': [], 'cNAPB': [], 'cNBN': [], 'cNBS': [],
-                          'cNBE': [], 'cNBC': [], 'cBdt': [], 'cNB': [],
-                          'cAPSummary': [],
-                          'cBSummary': []}
+        self.design_data = {'dNAPA': [], 'dNAPB': [], 'dNBN': [], 'dNBS': [],
+                            'dNBE': [], 'dNBC': [], 'dBdt': [], 'dNB': [],
+                            'dAPSummary': [],
+                            'dBSummary': []}
         self.test_procs = []
         master_path = (Path(apogee_data.__file__).absolute().parent.parent
-                           / "dat/master_dome_flat.fits.gz")
+                       / "dat/master_dome_flat.fits.gz")
         if not master_path.exists():
             master_path = (Path(apogee_data.__file__).absolute(
-                ).parent.parent.parent / "dat/master_dome_flat.fits.gz")
+            ).parent.parent.parent / "dat/master_dome_flat.fits.gz")
         master_data = fitsio.read(master_path.as_posix())
         self.ap_master = np.median(master_data[:, 550:910], axis=1)
         self.morning_filter = None
@@ -227,7 +229,7 @@ class Logging:
                 else:
                     i = self.ap_data['dDesign'].index(img.design_id)
                     self.ap_data["dConfig"][i] = min(self.ap_data["dConfig"][i],
-                                                  img.config_id)
+                                                     img.config_id)
                     if img.isot < self.ap_data['dTime'][i]:
                         self.ap_data['dTime'].pop(i)
                         self.ap_data['dTime'].insert(i, img.isot)
@@ -279,7 +281,7 @@ class Logging:
                     try:
                         self.data['cLead'].append(img.lead)
                     except AttributeError:
-                        self.data["cLead"].append("")    
+                        self.data["cLead"].append("")
                     self.data['dTime'].append(img.isot)
                 else:
                     i = self.data['dDesign'].index(img.design_id)
@@ -295,7 +297,7 @@ class Logging:
                 else:
                     i = self.b_data['dDesign'].index(img.design_id)
                     self.b_data["dConfig"][i] = min(self.b_data["dConfig"][i],
-                                                  img.config_id)
+                                                    img.config_id)
                     if img.isot < self.b_data['dTime'][i]:
                         self.b_data['dTime'].pop(i)
                         self.b_data['dTime'].insert(i, img.isot)
@@ -449,74 +451,74 @@ class Logging:
 
     def count_dithers(self):
         for i, design in enumerate(self.data['dDesign']):
-            self.design_data['cNAPA'].append(np.sum(
+            self.design_data["dNAPA"].append(np.sum(
                 (self.ap_data['iDesign'] == design)
                 & (self.ap_data['iDither'] == 'A')
                 & (self.ap_data['iEType'] == 'Object')))
-            self.design_data['cNAPB'].append(np.sum(
+            self.design_data["dNAPB"].append(np.sum(
                 (self.ap_data['iDesign'] == design)
                 & (self.ap_data['iDither'] == 'B')
                 & (self.ap_data['iEType'] == 'Object')))
-            # self.design_data['cNBN'].append(np.sum(
+            # self.design_data["dNBN'].append(np.sum(
             # (self.b_data['iDesign'] == design)
             # & (self.b_data['iDither'] == 'N')
             # & (self.b_data['iEType'] == 'Science')))
-            # self.design_data['cNBS'].append(np.sum(
+            # self.design_data["dNBS'].append(np.sum(
             # (self.b_data['iDesign'] == design)
             # & (self.b_data['iDither'] == 'S')
             # & (self.b_data['iEType'] == 'Science')))
-            # self.design_data['cNBE'].append(np.sum(
+            # self.design_data["dNBE'].append(np.sum(
             # (self.b_data['iDesign'] == design)
             # & (self.b_data['iDither'] == 'E')
             # & (self.b_data['iEType'] == 'Science')))
-            # self.design_data['cNBC'].append(np.sum(
+            # self.design_data["dNBC'].append(np.sum(
             # (self.b_data['iDesign'] == design)
             # & (self.b_data['iDither'] == 'C')
             # & (self.b_data['iEType'] == 'Science')))
-            self.design_data['cNB'].append(np.sum(
+            self.design_data["dNB"].append(np.sum(
                 (self.b_data['iDesign'] == design)
                 & (self.b_data['iEType'] == 'Science')))
-            if self.design_data['cNB'][-1] != 0:
-                self.design_data['cBdt'].append(np.max(
+            if self.design_data["dNB"][-1] != 0:
+                self.design_data["dBdt"].append(np.max(
                     self.b_data['idt'][
                         (self.b_data['iDesign'] == design)
                         & (self.b_data['iEType'] == 'Science')]))
             else:
-                self.design_data['cBdt'].append(0)
+                self.design_data["dBdt"].append(0)
 
         for i, design in enumerate(self.data['dDesign']):
             """To determine the number of apogee a dithers per design (cNAPA),
             as well as b dithers (cNAPB), and the same for NSE dithers."""
             # APOGEE dithers
-            if self.design_data['cNAPA'][i] == self.design_data['cNAPB'][i]:
-                if self.design_data['cNAPA'][i] != 0:
-                    self.design_data['cAPSummary'].append(
-                        '{}xAB'.format(self.design_data['cNAPA'][i]))
+            if self.design_data["dNAPA"][i] == self.design_data["dNAPB"][i]:
+                if self.design_data["dNAPA"][i] != 0:
+                    self.design_data['dAPSummary'].append(
+                        '{}xAB'.format(self.design_data["dNAPA"][i]))
                 else:
-                    self.design_data['cAPSummary'].append('No APOGEE')
+                    self.design_data['dAPSummary'].append('No APOGEE')
             else:
-                self.design_data['cAPSummary'].append(
-                    '{}xA {}xB'.format(self.design_data['cNAPA'][i],
-                                       self.design_data['cNAPB'][i]))
+                self.design_data['dAPSummary'].append(
+                    '{}xA {}xB'.format(self.design_data["dNAPA"][i],
+                                       self.design_data["dNAPB"][i]))
             # BOSS (MaNGA) dithers
-            # if self.design_data['cNBC'][i] == 0:
-            # if (self.design_data['cNBN'][i]
-            # == self.design_data['cNBS'][i]
-            # == self.design_data['cNBE'][i]):
-            # self.design_data['cBSummary'].append(
-            # '{}xNSE'.format(self.design_data['cNBN'][i]))
+            # if self.design_data["dNBC'][i] == 0:
+            # if (self.design_data["dNBN'][i]
+            # == self.design_data["dNBS'][i]
+            # == self.design_data["dNBE'][i]):
+            # self.design_data['dBSummary'].append(
+            # '{}xNSE'.format(self.design_data["dNBN'][i]))
             # else:
-            # self.design_data['cBSummary'].append(
-            # '{}xN {}xS {}xE'.format(self.design_data['cNBN'][i],
-            # self.design_data['cNBS'][i],
-            # self.design_data['cNBE'][i]))
+            # self.design_data['dBSummary'].append(
+            # '{}xN {}xS {}xE'.format(self.design_data["dNBN'][i],
+            # self.design_data["dNBS'][i],
+            # self.design_data["dNBE'][i]))
             # else:
-            if self.design_data['cNB'][i] != 0:
-                self.design_data['cBSummary'].append(
-                    '{}x{}s'.format(self.design_data['cNB'][i],
-                                    self.design_data['cBdt'][i]))
+            if self.design_data["dNB"][i] != 0:
+                self.design_data["dBSummary"].append(
+                    '{}x{}s'.format(self.design_data["dNB"][i],
+                                    self.design_data["dBdt"][i]))
             else:
-                self.design_data['cBSummary'].append('No BOSS')
+                self.design_data['dBSummary'].append('No BOSS')
 
     @staticmethod
     def hartmann_parse(hart):
@@ -543,11 +545,12 @@ class Logging:
         print('=' * 80)
         print('{:^80}'.format('Observing Summary'))
         print('=' * 80)
-        for i, design  in enumerate(self.data['dDesign']):
+        for i, design in enumerate(self.data['dDesign']):
             print('')
             print("Design {}, Config {}, {}, {}".format(design,
-                self.data['dConfig'][i], self.design_data['cAPSummary'][i],
-                self.design_data['cBSummary'][i]))
+                                                        self.data['dConfig'][i],
+                                                        self.design_data['dAPSummary'][i],
+                                                        self.design_data['dBSummary'][i]))
         print()
         if len(self.ap_data["fRatio"]) > 0:
             flux_ratio = np.nanmean(np.array(self.ap_data["fRatio"]), axis=0)
@@ -562,9 +565,9 @@ class Logging:
                 missing_bundles = self.ap_image.create_bundles(i_missing)
                 faint_bundles = self.ap_image.create_bundles(i_faint)
                 print("APOGEE Dome Flats\n"
-                    f"Missing Fibers: {missing_bundles}\n"
-                    f" Faint fibers: {faint_bundles}\n"
-                    f" Average Throughput: {avg:.3f}")
+                      f"Missing Fibers: {missing_bundles}\n"
+                      f" Faint fibers: {faint_bundles}\n"
+                      f" Average Throughput: {avg:.3f}")
                 print()
             else:
                 print("No APOGEE Dome Flats")
@@ -583,18 +586,21 @@ class Logging:
         # print()
 
     @staticmethod
-    def get_window(data, i):
+    def get_window(data, i, design):
         try:
             window = ((data['iTime']
                        >= data['dTime'][i])
                       & (data['iTime']
                          < data['dTime'][i + 1])
+                      & (data["iDesign"][i] == design)
                       )
 
         except IndexError:
             try:
                 window = ((data['iTime'] >= data['dTime'][i])
-                          & (data['iTime'] < Time.now() + 0.3))
+                          & (data['iTime'] < Time.now() + 0.3)
+                          & (data["iDesign"][i] == design)
+                         )
             except IndexError:
                 window = np.array([False] * len(data['iTime']))
 
@@ -606,7 +612,7 @@ class Logging:
         print('=' * 80 + '\n')
         for i, design in enumerate(self.data['dDesign']):
             print('### Design {}\n'.format(design))
-            if design  in self.ap_data['dDesign']:
+            if design in self.ap_data['dDesign']:
                 ap_design = np.where(design == self.ap_data['dDesign'])[0][0]
 
                 print('# APOGEE')
@@ -615,7 +621,8 @@ class Logging:
                                       'Dith', 'Reads', 'Arch',
                                       'Seeing'))
                 print('-' * 80)
-                window = self.get_window(self.ap_data, ap_design)
+                # window = self.get_window(self.ap_data, ap_design, design)
+                window = self.ap_data["iDesign"] == design
                 for (mjd, iso, conf, exp_id, exp_type, dith, nread,
                      detectors, see) in zip(
                     self.ap_data['iTime'][window].mjd + 0.3,
@@ -642,6 +649,8 @@ class Logging:
                                 self.ap_data['fMissing'][j]), 80))
                             print(textwrap.fill('Faint fibers: {}'.format(
                                 self.ap_data['fFaint'][j]), 80))
+                            print(f"Average Throughput:"
+                                  f" {np.nanmean(self.ap_data['fRatio'][j]):.2f}")
                             print()
 
             if design in self.b_data['dDesign']:
@@ -651,9 +660,10 @@ class Logging:
                                 'Dith', 'SOS', 'ETime', 'Hart'))
                 print('-' * 80)
                 # i is an index for data, but it will disagree with b_data
-                # if there is an apogee-onlydesign 
+                # if there is an apogee-onlydesign
                 b_design = np.where(design == self.b_data['dDesign'])[0][0]
-                window = self.get_window(self.b_data, b_design)
+                # window = self.get_window(self.b_data, b_design, design)
+                window = self.b_data["iDesign"] == design
                 for (mjd, iso, conf, exp_id, exp_type, dith,
                      detectors, etime, hart) in zip(
                     self.b_data['iTime'][window].mjd + 0.3,
@@ -672,22 +682,22 @@ class Logging:
                                     exp_type.strip(),
                                     dith.strip(), detectors, etime,
                                     hart))
-                try:
-                    window = ((self.b_data['hTime']
-                               >= self.data['dTime'][i])
-                              & (self.b_data['hTime']
-                                 < self.data['dTime'][i + 1])
-                              )
-                except IndexError:
-                    window = ((self.b_data['hTime']
-                               >= self.data['dTime'][i])
-                              & (self.b_data['hTime'] < Time.now()))
-                if self.b_data['hTime'][window]:
-                    print()
-                    # print('Hartmanns')
-                for t, hart in zip(self.b_data['hTime'][window],
-                                   self.b_data['hHart'][window]):
-                    print(self.hartmann_parse(hart))
+                #try:
+                #    window = ((self.b_data['hTime']
+                #               >= self.data['dTime'][i])
+                #              & (self.b_data['hTime']
+                #                 < self.data['dTime'][i + 1])
+                #              )
+                #except IndexError:
+                #    window = ((self.b_data['hTime']
+                #               >= self.data['dTime'][i])
+                #              & (self.b_data['hTime'] < Time.now()))
+                #if self.b_data['hTime'][window]:
+                #    print()
+                #    # print('Hartmanns')
+                #for t, hart in zip(self.b_data['hTime'][window],
+                #                   self.b_data['hHart'][window]):
+                #    print(self.hartmann_parse(hart))
                 print()
 
     def p_boss(self):
@@ -724,7 +734,7 @@ class Logging:
         print('=' * 80 + '\n')
         print('{:<5} {:<8} {:<13} {:<8} {:<12} {:<4} {:<5} {:<5}'
               ''.format('MJD', 'UTC', 'Design-Config', 'Exposure', 'Type',
-                              'Dith', 'Reads', 'Arch'))
+                        'Dith', 'Reads', 'Arch'))
         print('-' * 80)
         if self.args.morning:
             for (mjd, iso, design, config, exp_id, exp_type, dith, nread,
@@ -743,8 +753,8 @@ class Logging:
                       ' {:>5}'
                       ' {:<5}'
                       ''.format(int(mjd), iso[11:19], design, config,
-                                         exp_id, exp_type,
-                                         dith, nread, detectors))
+                                exp_id, exp_type,
+                                dith, nread, detectors))
 
         else:
             for (mjd, iso, design, config, exp_id, exp_type, dith, nread,
@@ -767,8 +777,8 @@ class Logging:
                       ' {:>5}'
                       ' {:<5}'
                       ''.format(int(mjd), iso[11:19], design, config,
-                                         exp_id, exp_type,
-                                         dith, nread, detectors))
+                                exp_id, exp_type,
+                                dith, nread, detectors))
 
         # Usually, there are 4 ThAr and 4 UNe arcs in a night, and they're
         # assumed to be alternating ThAr UNe ThAr UNe. When you grab every
@@ -786,7 +796,9 @@ class Logging:
         print('\n'.join(wrapper.wrap(une_str)))
         if len(self.ap_data['oOffset']) > 1:
             # Put it under an if in case we didn't open.
-            rel_offsets = np.abs(np.diff(self.ap_data['oOffset']))
+            did_move = self.ap_data["oDither"][:-1] != self.ap_data["oDither"][1:]
+            rel_offsets = np.abs(self.ap_data['oOffset'][1:][did_move]
+                                 - self.ap_data["oOffset"][:-1][did_move])
             obj_str = ('Object Offsets: Max: {:.2f}, Min: {:.2f}, Mean: {:.2f}'
                        ''.format(np.nanmax(rel_offsets), np.nanmin(rel_offsets),
                                  np.nanmean(rel_offsets)))
@@ -812,16 +824,29 @@ class Logging:
         end = Time(self.args.sjd + 1, format='mjd') - 0.3
         end = Time.now() if Time.now() < end else end
         tel = log_support.LogSupport(start, end, self.args)
+        support = multiprocessing.Manager().dict()
         tel.set_callbacks()
-        tel.get_offsets()
-        tel.get_focus()
-        tel.get_weather()
-        tel.get_hartmann()
-        print(tel.offsets)
-        print(tel.focus)
-        print(tel.weather)
-        print(tel.hartmann)
-
+        offsets = multiprocessing.Process(target=tel.get_offsets,
+                                          args=(support,))
+        focus = multiprocessing.Process(target=tel.get_focus,
+                                          args=(support,))
+        weather = multiprocessing.Process(target=tel.get_weather,
+                                          args=(support,))
+        hartmann = multiprocessing.Process(target=tel.get_hartmann,
+                                           args=(support,))
+        offsets.start()
+        focus.start()
+        weather.start()
+        hartmann.start()
+        offsets.join(10)
+        focus.join(10)
+        weather.join(10)
+        hartmann.join(10)
+        print(support["offsets"])
+        print(support["focus"])
+        print(support["weather"])
+        print(support["hartmann"])
+        
     @staticmethod
     def mirror_numbers():
         print('=' * 80)
@@ -899,6 +924,19 @@ def main():
     ap_data_dir = ap_dir / '{}'.format(args.sjd)
     b_data_dir = b_dir / '{}'.format(args.sjd)
     ap_images = Path(ap_data_dir).glob('apR-a*.apz')
+    try:
+        for img in ap_images:
+            img.exists()
+    except OSError:
+        pass
+    ap_images = Path(ap_data_dir).glob('apR-a*.apz')
+    
+    b_images = Path(b_data_dir).glob('sdR-r1*fit.gz')
+    try:
+        for img in b_images:
+            img.exists()
+    except OSError:
+        pass
     b_images = Path(b_data_dir).glob('sdR-r1*fit.gz')
 
     if not args.noprogress:
@@ -918,8 +956,8 @@ def main():
         args.apogee = True
         p_boss = True
         p_apogee = True
-        # args.log_support = True  # Because this usually produces wrong results
-        args.mirrors = True
+        args.log_support = True  # Because this usually produces wrong results
+        # args.mirrors = True
         args.telstatus = True
 
     if args.summary:
@@ -955,8 +993,8 @@ def main():
         log.log_support()
 
     if args.mirrors:
-        pass
-        # log.mirror_numbers()
+        # pass
+        log.mirror_numbers()
 
     if args.telstatus:
         log.tel_status()
