@@ -6,6 +6,7 @@ STUI, but by bypassing STUI and directly accessing telemetry
 TODO: Thread these queries
 """
 import argparse
+import multiprocessing
 
 import numpy as np
 
@@ -17,11 +18,47 @@ from bin import influx_fetch, sjd
 __version__ = '3.3.0'
 
 
+def get_boss_callbacks(tstart, tend, call_dict):
+    out = []
+    boss_exp_path = Path(__file__).parent.parent / "flux/science_exposures.flux"
+    with boss_exp_path.open('r') as fil:
+        tables = influx_fetch.query(fil.read(), tstart, tend)
+    for table in tables:
+        for row in table.records:
+            out.append(row.get_time())
+    
+    call_dict["boss_calls"] = out
+
+def get_apogee_callbacks(tstart, tend, call_dict):
+    out = []
+    apog_exp_path = Path(__file__).parent.parent / "flux/apogee_science.flux"
+    with apog_exp_path.open('r') as fil:
+        tables = influx_fetch.query(fil.read(), tstart, tend)
+    for table in tables:
+        for row in table.records:
+            out.append(row.get_time())
+    call_dict["apogee_calls"] = out
+    
+def get_enclosure_history(tstart, tend, call_dict):
+    out_times = []
+    out_states = []
+    flx_path = Path(__file__).parent.parent / "flux/enclosure.flux"
+    with flx_path.open('r') as fil:
+        tables = influx_fetch.query(fil.read(), tstart, tend)
+    for table in tables:
+        for row in table.records:
+            out_times.append(row.get_time())
+            out_states.append(row.get_value())
+    call_dict["enclosure_times"] = out_times
+    call_dict["enclosure_states"] = out_states
+ 
 class LogSupport:
     def __init__(self, tstart, tend, args):
         self.tstart = Time(tstart)
         self.tend = Time(tend)
         self.args = args
+        if args.verbose:
+            print(f"Log Support Window: {self.tstart}-{self.tend}")
 
         self.call_times = []
         self.offsets = ""
@@ -29,11 +66,53 @@ class LogSupport:
         self.weather = ""
         self.hartmann = ""
 
+       
     def set_callbacks(self):
-        self.call_times = Time(np.arange((self.tstart + 0.3).mjd, self.tend.mjd,
-                15 * 60 / 86400), format="mjd")
+        callback_dict = multiprocessing.Manager().dict()
+        boss = multiprocessing.Process(target=get_boss_callbacks,
+                                       args=(self.tstart, self.tend,
+                                             callback_dict,))
+        apogee = multiprocessing.Process(target=get_apogee_callbacks,
+                                       args=(self.tstart, self.tend,
+                                             callback_dict,))
+        enclosure = multiprocessing.Process(target=get_enclosure_history,
+                                       args=(self.tstart, self.tend,
+                                             callback_dict,))
+        boss.start()
+        apogee.start()
+        enclosure.start()
+        boss.join(5)
+        apogee.join(5)
+        enclosure.join(5)
+        try:
+            all_times = Time(callback_dict["boss_calls"]
+                             + callback_dict["apogee_calls"])
+        except ValueError:
+            self.call_times = []
+            return
+        all_times = all_times[all_times.argsort()]
+        encl_times = Time(callback_dict["enclosure_times"])
+        encl_vals = np.array(callback_dict["enclosure_states"])
+        encl_vals = encl_vals[encl_times.argsort()]
+        encl_times = encl_times[encl_times.argsort()]
+        new_times = []
+        for time in all_times:
+            before = encl_times < time
+            if before.sum() == 0:
+                continue
+            if encl_vals[before][-1] != 1:
+                continue
+            if np.all([time - (15 / 24 / 60) > t for t in new_times]):
+                new_times.append(time)
+        
+        try:
+            self.call_times = Time(new_times)
+        except ValueError:
+            self.call_times = Time(new_times, format="mjd")
+        # self.call_times = Time(np.arange((self.tstart + 0.3).mjd, self.tend.mjd,
+                # 15 * 60 / 86400), format="mjd")
 
-    def get_offsets(self):
+    def get_offsets(self, out_dict={}):
         self.offsets = f"{'Time':<8} {'Az':<6} {'Alt':<4} {'Rot':<6}\n"
         self.offsets += '=' * 80 + '\n'
         offsets_tab = {}
@@ -68,18 +147,9 @@ class LogSupport:
             if np.all(np.isnan(np.array(line[1:]))):
                 continue
             self.offsets += "{:<8} {:>6.1f} {:>4.1f} {:>6.1f}\n".format(*line)
-                
-            
-        # offsets_tab_keys = ['25m:guider:cartridgeLoaded:cartridgeID',
-        #                 '25m:guider:cartridgeLoaded:plateID',
-        #                 '25m:guider:survey:plateType',
-        #                 '25m:tcc:axePos:az', '25m:tcc:axePos:alt',
-        #                 '25m:tcc:axePos:rot', '25m:tcc:objArcOff:az',
-        #                 '25m:tcc:objArcOff:alt', '25m:tcc:guideOff:rot',
-        #                 '25m:tcc:calibOff:az', '25m:tcc:calibOff:alt',
-        #                 '25m:tcc:calibOff:rot', '25m:guider:guideRMS:RMSerror']
+        out_dict["offsets"] = self.offsets        
 
-    def get_focus(self):
+    def get_focus(self, out_dict={}):
         self.focus = (f"{'Time':<8} {'M1':<4} {'M2':<4} {'Focus':<5}"
                       f" {'Az':<6} {'Alt':<5} {'Temp':<5} {'Wind':<4}"
                       f" {'Dir':<3}\n")
@@ -121,55 +191,9 @@ class LogSupport:
             self.focus += ("{:<8} {:>4.0f} {:>4.0f} {:>5.0f} {:>6.1f} {:>5.1f}"
                            " {:>5.1f} {:>4.0f} {:>3.0f}\n".format(
                                  *line))
-                
-        # focus_keys = ['25m:guider:cartridgeLoaded:cartridgeID',
-        #               '25m:guider:cartridgeLoaded:plateID',
-        #               '25m:guider:survey:plateType',
-        #               '25m:tcc:scaleFac', '25m:tcc:primOrient:pos',
-        #               '25m:tcc:secOrient:piston', '25m:tcc:secFocus',
-        #               '25m:tcc:axePos:az', '25m:tcc:axePos:alt',
-        #               '25m:apo:airTempPT', '25m:apo:winds',
-        #               '25m:apo:windd', '25m:guider:seeing']
-        # foc_data = {}
-        # for key in focus_keys:
-        #     foc_data[key] = []
-        # for time in self.call_times:
-        #     self.query(focus_keys, time.datetime, time.datetime,
-        #                foc_data)
+        out_dict["focus"] = self.focus
 
-        # self.focus += '=' * 80 + '\n'
-        # self.focus += '{:^80}\n'.format('Telescope Focus')
-        # self.focus += '=' * 80 + '\n\n'
-        # self.focus += ('{:<5} {:<9} {:<6} {:<5} {:<5} {:<5} {:<6} {:<5}'
-        #                ' {:<5} {:<4} {:<3}'
-        #                ' {:<4}\n'.format('Time', 'Cart', 'Scale', 'M1', 'M2',
-        #                                  'Focus', 'Az', 'Alt', 'Temp',
-        #                                  'Wind', 'Dir', 'FWHM'))
-        # self.focus += '-' * 80 + '\n'
-        # for i, time in enumerate(self.call_times):
-        #     if foc_data[focus_keys[2]][i] == '':
-        #         continue
-        #     self.focus += ('{:>5} {:>2}-{:0>5}{:>1} {:>6.1f} {:>5.0f}'
-        #                    ' {:>5.0f}'
-        #                    ' {:>5.0f} {:>6.1f} {:>5.1f} {:>5.1f} {:>4.0f}'
-        #                    ' {:>3.0f}'
-        #                    ' {:>4.1f}\n'.format(time.isot[11:16],
-        #                                         foc_data[focus_keys[0]][i],
-        #                                         foc_data[focus_keys[1]][i],
-        #                                         foc_data[focus_keys[2]][i][0],
-        #                                         (foc_data[focus_keys[3]][i]
-        #                                          - 1) * 1e6,
-        #                                         foc_data[focus_keys[4]][i],
-        #                                         foc_data[focus_keys[5]][i],
-        #                                         foc_data[focus_keys[6]][i],
-        #                                         foc_data[focus_keys[7]][i],
-        #                                         foc_data[focus_keys[8]][i],
-        #                                         foc_data[focus_keys[9]][i],
-        #                                         foc_data[focus_keys[10]][i],
-        #                                         foc_data[focus_keys[11]][i],
-        #                                         foc_data[focus_keys[12]][i]))
-
-    def get_weather(self):
+    def get_weather(self, out_dict={}):
         dust = "1\u03BCm Dust"
         irscs = "IRSC \u03C3"
         irscm = "IRSC \u03BC"
@@ -191,14 +215,28 @@ class LogSupport:
                 else:
                     weather_tab[f"t{measurement}"] = [row.get_time()]
                     weather_tab[measurement] = [row.get_value()]
+                    
+        # Filter out dpDep values by adding a fake value every time the humidity
+        # is above 90
+        for i, h in enumerate(weather_tab["humidPT"]):
+            if h > 90:
+                # print(weather_tab["thumidPT"][i], h)
+                weather_tab["dustb"].append(np.nan)
+                weather_tab["tdustb"].append(weather_tab["thumidPT"][i])
         for key in weather_tab.keys():
             if key[0] == 't':
                 weather_tab[key] = Time(weather_tab[key])
             else:
                 weather_tab[key] = np.array(weather_tab[key])
+        # Most data is assumed to be sorted, but because of the dewpoint filter
+        # for dust, we need to resort it to put the added values in their
+        # proper places
+        weather_tab["dustb"] = weather_tab["dustb"][
+            weather_tab["tdustb"].argsort()]
+        weather_tab["tdustb"] = weather_tab["tdustb"][
+            weather_tab["tdustb"].argsort()]
         for t in self.call_times:
             line = [t.isot[11:19]] 
-            skip_line = False
             for key in ["airTempPT", "airTempPT", "dpTempPT", "humidPT", "winds", "windd",
                         "dustb", "irscsd", "irscmean"]:
                 if 't' + key in weather_tab.keys():
@@ -218,8 +256,9 @@ class LogSupport:
             self.weather += ("{:<8} {:>5.1f} {:>5.1f} {:>5.1f} {:>5.1f} {:>5.1f}"
                              " {:>3.0f} {:>8.0f} {:>6.1f} {:>6.0f}\n".format(
                                  *line))
+        out_dict["weather"] = self.weather
 
-    def get_hartmann(self):
+    def get_hartmann(self, out_dict={}):
         self.hartmann = f"{'Time':8} {'Temp':<6} {'R off':<6} {'B off':<6}"
         self.hartmann += f" {'Move':<6} {'Resid':<6}\n"
         self.hartmann += '=' * 80 + '\n'
@@ -265,6 +304,7 @@ class LogSupport:
                 line.append(harts[key][in_window][0])
             self.hartmann += ("{:8} {:>6.1f} {:>6.0f} {:>6.1f} {:>6.0f}"
                               " {:>6.1f}\n".format(*line))
+        out_dict["hartmann"] = self.hartmann
 
 def main():
     parser = argparse.ArgumentParser()
